%% Two simple examples
%

%% Example 1: two sets of intersecting lines
% In the first example, we solve the pair of equations
% $$ x^2-1 = 0 $$
% $$ y^2-4 =0. $$
% The solution set consists of the four points $ (\pm 1,\pm 2). $ The
% |BertiniLab| object for this problem can be constructed as follows:
poly_system = BertiniLab('variable_group',{'x','y'},'function_def',{'x^2-1'; 'y^2-4'});

%%
% To solve the system, simply type
poly_system = poly_system.solve;

%%
% Note that |poly_system| appears as an output. This allows it to store two
% important results of the run. The first is the property
% |order_of_variables|, which  keeps track of the order in which
% the variables were declared in the Bertini input file. This is important
% for retrieving the solutions later and assigning them to the correct
% variables. The second is |solve_summary|, which captures the
% output that normally goes to the screen:
disp(poly_system.solve_summary)


%%
% This summary reports that four paths were tracked and there are four
% non-singular, finite solutions of multiplicity one. They are also real,
% so the same solutions appear in the Bertini output files |raw_solutions|,
% |nonsingular_solutions|, |finite_solutions| and |real_finite_solutions|.
%
% |BertiniLab| has two methods for retrieving the output from a Bertini
% file. The first, |read_raw_data|, reads the file |raw_data|, which
% contains both the solution points and a set of diagnostics for each point.
% The other files can be read using |match_solutions|. This reads a set of
% solutions and matches it to the appropriate variables.
sols = poly_system.match_solutions('real_finite_solutions');
disp(sols)

%%
% The variable |sols| is a structure with a field for each variable. The
% solutions are stored as |polysym| objects, which can be converted to
% double precision numbers, cell arrays of strings, sym or vpa objects.
x = real(double(sols.x));
y = real(double(sols.y));
disp([x(:) y(:)])

%% Example 2: a system with singular points
%
% The second example solves the system
%
% $$ (29/16)*z1^3 - 2*z1*z2 = 0 $$
%
% $$ z2 - z1^2 = 0. $$
%
% Begin by defining the variables as |polysym| objects:
polysyms z1 z2

%%
% When we use |polysym| objects for the variables, we can make use of a
% MATLAB function to create the equations. We define the anonymous
% function:
f = @(x,y) [(29/16)*x^3 - 2*x*y; y - x^2];

%%
% Create the BertiniLab object. The variables need to be declared in a
% |variable_group|, |hom_variable_group| or |variable|, depending on the
% application (the most common choice is |variable_group|).
poly_system = BertiniLab('function_def',f(z1,z2),'variable_group',[z1 z2]);

%%
% Note that when the function is |polsym| arguments, the square brackets
% collect the output in a vector of class |polysym|. There should be a
% semcolon between each expression, creating a row vector.
f = [(29/16)*z1^3 - 2*z1*z2; z2 - z1^2];

%%
% Solve the system and save some information in |poly_system|:
poly_system = poly_system.solve;

%%
% The property |solve_summary| contains the output that would go to the
% screen in an interactive Bertini run. The last lines of the output
% contain a warning:
results = poly_system.solve_summary;
istart = strfind(results,'Truncated');
disp(results(istart:end))

%%
% Taking the advice of Bertini, we set SecurityLevel to 1 and solve again:
poly_system.config = struct('SecurityLevel',1);
poly_system = poly_system.solve;

%%
% The summary tables are shown below:
results = poly_system.solve_summary;
istart = strfind(results,'Tracking path');
iend = strfind(results,'The following');
disp(results(istart:iend-1))
 
%%
% In this example, the variables |z1| and |z2| are declared in the same
% variable group. However, it happens that a multihomogenous homotopy,
% where |z1| and |z2| are in separate variable groups, needs to explore
% fewer paths. This can be set up by collecting |z1| and |z2| in curly
% brackets. The result is a cell array of |polysym| objects instead of a
% |polysym| object:
disp(class([z1 z2]))
disp(class({z1,z2}))

%%
% Now for the solution:
config = struct('SecurityLevel',1);
poly_system = BertiniLab('function_def',f,'variable_group',{z1,z2},'config',config);
poly_system = poly_system.solve;
results = poly_system.solve_summary;
istart = strfind(results,'Paths Tracked');
disp(results(istart:end))

%%
% The result is a (not very dramatic) reduction in effort from 6 paths to
% 5.

%%
% The solutions are stored as |polysym|, allowing them to represent
% variable precision. They can be converted to double precision:
sols = poly_system.match_solutions('finite_solutions');
fprintf('%12s %18s\n','z1','z2');
fprintf('%15.11g %17.11g\n',double([sols.z1 sols.z2]).')
