%% Examples from the Bertini Users Manual
clear classes

%% Section 2.2.1: Zero-dimensional examples
%
% This includes an illustration of how to set up multiple VARIABLE_GROUP
% declarations: Use a cell array.
config = struct('TRACKTOLBEFOREEG',1E-8,'TRACKTOLDURINGEG',1E-11,'FINALTOL',1E-14);
z = polysym('z',[1 2]);
f = polysym('f',[1 2]);
vargroup = {z(1),z(2)};

fvals = [(29/16)*z(1)^3 - 2*z(1)*z(2); z(2)-z(1)^2];
fDef = [f(:) fvals(:)];

obj = BertiniLab('config',config,'variable_group',vargroup,'function_def',fDef);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 2.2.3: User-defined homotopies

config = struct('USERHOMOTOPY',1);

polysyms f x t s
gammaLHS = polysym('gamma'); % use gammaLHS instead of builtin name 'gamma'
gammaRHS = polysym('0.8-1.2*I');

obj = BertiniLab('config',config,'variable',x,'pathvariable',t,'parameter', ...
    [s t],'constant',[gammaLHS gammaRHS],'init_values',{x,[-1 1]});

obj.function_def = [f (x^2 - 1)*gammaLHS*s + (x^2)*(1-s)];

obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.1: Basic example (Input file: Fig. 4.1)
%
% This is also an example of a different way of inputting data to
% BertiniLab. This only works if there are no user-defined constants (but
% i, j, pi, inf, nan, eps are o.k.)

eqns = {'x^2-1'; 'x+y-1'}; % Note the semicolon: this must be a column vector.
vars = symvar([eqns{:}]);  % MATLAB command that finds the variables in a string.

obj = BertiniLab('function_def',eqns,'variable_group',vars);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.1: Basic example with sharpening (Input file: Fig. 4.4)
polysyms x y f g
config = struct('SHARPENDIGITS',20);

obj = BertiniLab('function_def',[f x^2-1; g x+y-1],'variable_group',[x y], ...
    'config',config);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.2: Extended example 1 (Input file: Fig. 4.6)
polysyms x y f g

obj = BertiniLab('function_def',[f x^2-1; g x+y-1],'variable_group',[x y]);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.2: Extended example 2 (Input file: Fig. 4.8)
polysyms x y f g
config = struct('SECURITYLEVEL',1,'TRACKTOLBEFOREEG',1e-7, ...
    'TRACKTOLDURINGEG',1e-7,'FINALTOL',1e-12,'COEFFBOUND',2,'DEGREEBOUND',2);

obj = BertiniLab('function_def',[f x^2-1; g x+y-1],'variable_group',[x y],'config',config);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.2: Extended example 3 (Input file: Fig. 4.10)
polysyms x y f g

obj = BertiniLab('function_def',[f x^2-1; g x+y-1],'variable_group',{x, y});
obj = solve(obj);
disp(obj.solve_summary)

%% Section 4.4.3: Regeneration example (Input file: Fig. 4.12)
polysyms x y z S
subfunction = [S x^2+y^2+z^2-1];
fname = polysym('f',[3 1]);
fvals = [(y-x^2)*S*(x-0.5); (z-x^3)*S*(y-0.5); (y-x^2)*(z-x^3)*S*(z-0.5)];
config = struct('USEREGENERATION',1);

obj = BertiniLab('function_def',[fname fvals],'config',config, ...
    'variable_group',[x y z],'subfunction',subfunction);
obj = solve(obj);
disp(obj.solve_summary)

%% Section 5.4.1: Basic positive-dimensional example (Input file: Fig. 5.1)
polysyms x y z f
fvals = x^2+y^2+z^2-1;
config = struct('TRACKTYPE',1);

obj = BertiniLab('function_def',[f fvals],'variable_group',[x y z],'config',config);
obj = solve(obj);
disp(obj.solve_summary)

%% Twisted cubic in P^3 (Input file: Fig. 5.3)
polysyms w x y z
fvals = [x^2 - w*y; x*y - w*z; x*z-y^2];
fname = polysym('f',size(fvals));
config = struct('TRACKTYPE',1,'WITNESSGENTYPE',1);

obj = BertiniLab('function_def',[fname(:) fvals(:)],'hom_variable_group',[w x y z],'config',config);
obj = solve(obj);
disp(obj.solve_summary)


%% Section 5.4.3 Deflation example (Input file: Fig. 5.5)
polysyms x y
f = @(a,b) [polysym(29)/16*a^3-2*a*b; y-x^2]; % Note polysym(29)
fname= polysym('f',[1 2]);
fvals = f(x,y);
config = struct('TRACKTYPE',1,'SHARPENDIGITS',30);

obj = BertiniLab('function_def',[fname(:) fvals(:)],'config',config, ...
    'variable_group',[x y]);
obj = solve(obj);
disp(obj.solve_summary)
