%% Generalized eigenvalue problem: An illustration of ways of using BertiniLab
%
% These example show how BertiniLab can be used to call Bertini in a
% variety of ways. The problem is to solve the generalized eigenvalue
% problem
%
% $$\mu A v = \lambda B v. $$
%
% The first example enters the information much as it appears in the input
% file. In the second, a function is used to create the polynomial
% equations. In the third, a subclass of BertiniLab is created.

%% As in Bertini book (example 4.2)
%
% In this example we define variables and write the equations out as in the
% example in the book. However, it is not necessary to provide the names of
% the functions, unless you happen to have variables with names like
% 'eqn1', 'eqn2', ...
%
% If you input strings, use curly brackets (cell array). Since |mu| is a
% MATLAB function, we will call the variable |mu0|.
%
% In this example, there are two groups of When 
eqns = {'mu0*(1*v1 + 2*v2) - lambda*( 4*v1 - 2*v2)'; 'mu0*(2*v1 + 4*v2) - lambda*(-2*v1 + 1*v2)'};

%%
% There are two groups of homogeneous variables. When they are input as
% strings, each group should be in its own cell array (e.g.,
% |{'mu0','lambda'}|, and the arrays should be enclosed in curly brackets.
hom_group_1 = {'mu0','lambda'};
hom_group_2 = {'v1','v2'};
poly_system = BertiniLab('function_def',eqns, ...
    'hom_variable_group',{hom_group_1,hom_group_2});
poly_system = solve(poly_system);

%%
% Here is the summary of the Bertini run:
disp(poly_system.solve_summary)

%%
% The solutions can be retrieved from the file |raw_solutions| and paired
% with the variable names using |match_solutions|. Although |v1| and |v2|
% are components of a vector, they are stored independently.
sols = poly_system.match_solutions('raw_solutions');
mu0sols = double(sols.mu0); lambdasols = double(sols.lambda); 
v1sols = double(sols.v1); v2sols = double(sols.v2);
fprintf('%18s %35s %33s %33s\n','mu','lambda','v1','v2')
fprintf('%15.11f + %15.11fi %15.11f + %15.11fi %15.11f + %15.11fi %15.11f + %15.11fi\n', ...
    [real(mu0sols) imag(mu0sols) real(lambdasols) imag(lambdasols) real(v1sols) imag(v1sols) real(v2sols) imag(v2sols)].')

%% Using a pre-defined function
% We suppose that a function has already been created for this problem and
% we wish to find its zeros. We can now use the more natural representation
% with matrices |A|, |B| and vector |v|.
f = @(A,B,mu0,lambda,v) (mu0*A-lambda*B)*v.';

polysyms mu0 lambda;

%%
% We choose A and B to solve the same problem as in the previous example. A
% and B can be in a numeric class (in this example, they are integers
% stored as double precision); when combined with polysym objects, the
% result is a polysym object.
A = [1 2; 2 4]; B = [4 -2; -2 1];

%%
% We generalize the solution so that the eigenvalues could be obtained for
% a problem with a different number of dimensions.
N = length(A);

%%
% Create the BertiniLab object and solve.
fname = polysym('f',[1 N]);
v = polysym('v',[1 N]);

%%
% Note that each variable group is now a |polysym| array (enclosed in
% square brackets).
poly_system = BertiniLab('function_def',f(A,B,mu0,lambda,v), ...
    'hom_variable_group',{[mu0 lambda],v});
poly_system = solve(poly_system);

%%
% Now we can match the solutions to the vector variable |v|:
sols = poly_system.match_solutions('raw_solutions',v);
disp(sols.v)

%% Using a subclass
%
% In this example we create a subclass called
% |generalized_eigenvalue_problem| with a constructor that requires inputs
% |A| and |B|.
A = [1 2; 2 4]; B = [4 -2; -2 1];

poly_system = ge_problem(A,B);
poly_system = poly_system.solve;

disp(poly_system.eigenvalue_pairs)